/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core.annotator;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.extended.exception.WebLabUncheckedException;
import org.ow2.weblab.core.extended.factory.WebLabResourceFactory;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.rdf.RDFReader;
import org.ow2.weblab.rdf.RDFWriter;
import org.ow2.weblab.rdf.Value;
import org.ow2.weblab.rdf.XMLParser;

/**
 * Defines base methods to manage annotations on resources.
 * It provides the following functionalities:
 * <ul>
 * <li>Read statement</li>
 * <li>Add new statement</li>
 * <li>Overwrite statement</li>
 * <li>Delete statement</li>
 * </ul>
 * 
 * It can switch from an <code>append mode</code> to an <code>overwrite mode</code> to either add new statements or overwrite existing ones, see <code>setAppendMode(boolean)</code>.
 * 
 * To add a statement with a given language, define Annotator language with <code>startSpecifyLanguage(String lang)</code>.
 * 
 * To change the subject of the annotation, use  <code>startInnerAnnotatorOn(URI subject)</code>.
 * 
 * @author asaval
 *
 */
public abstract class AbstractAnnotator {


	/**
	 * The element properties used to write statements in.
	 */
	private Element element = Element.getDefaultElement();


	/**
	 * Operator to select action
	 * @author asaval
	 *
	 */
	public static enum Operator {
		ADD, REMOVE, SET, READ, WRITE
	}


	/**
	 * The Pok on which we will write and read statements
	 */
	private List<PieceOfKnowledge> poks;


	/**
	 * The WebLab Resource that will be annotated.
	 * It can be null since we can annotate subject which are just URI
	 * 
	 * @see #AbstractAnnotator(URI, PieceOfKnowledge)
	 */
	private Resource resource;


	/**
	 * The subject URI that will be used when writing/reading a statement.
	 */
	private URI subjectURI;


	/**
	 * State of the inner Annotator
	 */
	private boolean innerAnnotatorStarted = false;


	/**
	 * The original URI from the constructor (either subject or Resource's uri)
	 */
	private URI originalURI;


	/**
	 * The language currently supported in annotation
	 * If null, all languages are retrieve
	 */
	private String lang = null;


	/**
	 * Set the writing mode of the annotator (ADD/SET)
	 */
	private boolean append = true;


	/**
	 * Creates an annotator on the Resource.
	 * It allows to write and read statement (subject, property, object) about this resource on this resource.
	 * 
	 * @param resource the resource to wrap.
	 */
	protected AbstractAnnotator(final Resource resource) {
		this.resource = resource;
		try {
			this.subjectURI = new URI(resource.getUri());
		} catch (final URISyntaxException e) {
			LogFactory.getLog(BaseAnnotator.class).error("Illegal resource URI " + resource.getUri(), e);
		}
	}


	/**
	 * Creates an annotator for an URI.
	 * It allows to write and read about the subject in this PoK
	 * 
	 * @param subject
	 *            the subject of interest
	 * @param pok
	 *            the PieceOfKnowledge on which we manage information about the subject
	 */
	protected AbstractAnnotator(final URI subject, final PieceOfKnowledge pok) {
		this.innerAnnotatorStarted = true;
		this.resource = null;
		this.subjectURI = subject;
		this.originalURI = subject;
		if (pok != null) {
			this.poks = new LinkedList<PieceOfKnowledge>();
			this.poks.add(pok);
		}
	}

	/**
	 * Returns the current subject
	 * 
	 * @return the current subject
	 */
	public URI getSubject() {
		return this.subjectURI;
	}
	
	/**
	 * Returns the current resource
	 * 
	 * @return the current resource
	 */
	protected Resource getResource() {
		return this.resource;
	}


	/**
	 * Setter for current resource
	 * 
	 * @param resource
	 *            update the current resource
	 */
	protected void setResource(final Resource resource) {
		this.resource = resource;
	}


	/**
	 * Set the element that will contain statements.
	 * By default, the element is an RDF:Description object
	 * 
	 * @param prefix
	 *            the prefix of the element
	 * @param uri
	 *            the uri of the element
	 * @param localName
	 *            the localName of the element
	 */
	public void setElement(final String prefix, final String uri, final String localName) {
		this.element = new Element(prefix, uri, localName);
	}


	/**
	 * Set the element that will contain statements.
	 * By default, the element is an RDF:Description object
	 * 
	 * @param element
	 *            an Element
	 */
	public void setElement(final Element element) {
		this.element = element;
	}


	/**
	 * Set the default statements container that is an RDF:Description object
	 */
	public void setDefaultElement() {
		this.element = Element.getDefaultElement();
	}


	/**
	 * Start a inner annotator on a different subject.
	 * It allows to use an annotator to read/write stament about on other subject.
	 * If an inner annotator is already started, it will be automatically closed
	 * 
	 * @param subject
	 */
	public void startInnerAnnotatorOn(final URI subject) {
		this.innerAnnotatorStarted = true;
		this.subjectURI = subject;
	}


	/**
	 * End a inner annotator, the subject will be reseted to the first one
	 * (retrieved when the Annotator has been constructed).
	 */
	public void endInnerAnnotator() {
		if (this.resource == null) {
			this.subjectURI = this.originalURI;
		} else {
			this.innerAnnotatorStarted = false;
			this.subjectURI = URI.create(this.resource.getUri());
		}
	}


	/**
	 * Return the writing mode of the annotator:
	 * if the annotator is in append mode, it will add annotations else the annotator will override existing annotations
	 * 
	 * @return true if the annotator will add annotations or false if the annotator will override existing annotations
	 */
	public boolean isAppendMode() {
		return this.append;
	}


	/**
	 * Set the writing mode of the annotator.
	 * If the annotator is in append mode setAppendMode(true), then writing null value will fail and the annotator will do nothing. In this case, the returned
	 * POK will be null.
	 * If the annotator is in override mode setAppendMode(false), then writing null value will delete all the statements with the property ONLY in the FIRST
	 * annotation of the resource.
	 * 
	 * @param appendMode
	 *            true if the annotator will add annotations or false if the annotator will override existing annotations
	 * 
	 */
	public void setAppendMode(final boolean appendMode) {
		this.append = appendMode;
	}


	/**
	 * Start to support a specific language defined annotation
	 * 
	 * @param language
	 */
	public void startSpecifyLanguage(final String language) {
		this.lang = language;
	}


	/**
	 * End the support for specific language
	 */
	public void endSpecifyLanguage() {
		this.lang = null;
	}


	/**
	 * Returns true if inner annotator is started else false
	 * 
	 * @return true if inner annotator is started else false
	 */
	public boolean isInnerAnnotatorStarted() {
		return this.innerAnnotatorStarted;
	}


	protected <T> void retrieveAnnotation(final Operator operator, final T value) {
		if ((this.poks != null) && !this.poks.isEmpty() && this.innerAnnotatorStarted) {
			return;
		}
		this.poks = new LinkedList<PieceOfKnowledge>();
		if (this.resource.getAnnotation().isEmpty() && operator != AbstractAnnotator.Operator.READ && value != null){
				this.poks.add(WebLabResourceFactory.createAndLinkAnnotation(this.resource));
		} else {
			this.poks.addAll(this.resource.getAnnotation());
		}
	}


	/**
	 * Apply given operator to the resource
	 * @param operator the operator
	 * @param uriPrefix the prefix
	 * @param propertyUri the namespace of the property
	 * @param localname the localname of the property
	 * @param clazz the class of the value
	 * @param val the value
	 * @return a value containing result of operation.  
	 */
	public <T> Value<T> applyOperator(final Operator operator, final String uriPrefix, final URI propertyUri, final String localname, final Class<T> clazz,
			final T val) {
		Value<T> value = null;
		Operator finalop = operator;
		if (operator == AbstractAnnotator.Operator.WRITE && !this.append && val == null){
			finalop = AbstractAnnotator.Operator.REMOVE;
		}
		this.retrieveAnnotation(operator, val);
		
		switch (finalop) {
			case READ:
				for (final PieceOfKnowledge pok : this.poks) {
					final Value<T> ivalue = RDFReader.readValueFromStatement(pok, this.subjectURI, propertyUri, localname, this.lang, clazz, this.element);
					if (ivalue != null) {
						value = ivalue;
					}
				}
				break;
			case WRITE:

				try {
					final Object object = XMLParser.xmlTransform(val);
					final PieceOfKnowledge pok = this.poks.get(0);
					RDFWriter.writeStatement(pok, this.subjectURI, uriPrefix, propertyUri, localname, object, this.lang, this.append, this.element);
					value = new Value<T>(val, pok);

				} catch (final Exception e) {
					LogFactory.getLog(AbstractAnnotator.class).error(
							"Could not write property " + propertyUri + localname + " on " + this.subjectURI + " with value " + val, e);
					value = new Value<T>(val, null); // the POK is set to null since the property an not been written nor deleted
				}
				break;
			case ADD:
				final Object object = XMLParser.xmlTransform(val);
				final PieceOfKnowledge pok = this.poks.get(0);
				RDFWriter.removeStatement(pok, this.subjectURI, uriPrefix, propertyUri, localname, object, this.lang, this.append, this.element);
				break;
			case REMOVE:
				final Object object1 = XMLParser.xmlTransform(val);
				for (final PieceOfKnowledge pok1 : this.poks) {
					RDFWriter.removeStatement(pok1, this.subjectURI, uriPrefix, propertyUri, localname, object1, this.lang, this.append, this.element);
				}
				break;
			default:
				throw new WebLabUncheckedException("Operator " + operator + " was not taken into account.");
		}

		if (value == null) {
			value = new Value<T>();
		}

		return value;
	}


}
