/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.rdf;

import java.lang.instrument.IllegalClassFormatException;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.logging.LogFactory;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;
/**
 * XMLParser class managed type parser to transform String value into an other Type of Object.
 * By default the following types are supported:
 * <ul>
 * <li>Boolean</li>
 * <li>Byte</li>
 * <li>Integer</li>
 * <li>Double</li>
 * <li>Float</li>
 * <li>Long</li>
 * <li>Character</li>
 * <li>Date</li>
 * <li>String</li>
 * </ul>
 * @author asaval
 *
 */
public final class XMLParser {

	private static Map<Class<?>, AbstractTypeParser<?>> parsers = resetParsers(); 
	
	private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ", Locale.ENGLISH);

	private XMLParser() {
		throw new UnsupportedOperationException("This class should not been instanciated.");
	}

	/**
	 * Parse the value of a property info an instance of T.
	 * @param <T> the type the value should be returned
	 * @param value the value that must be converted into type
	 * @param clazz the class of the type
	 */
	@SuppressWarnings("unchecked")
	protected static <T> T xmlParse(final String value, final Class<T> clazz) {
		T result = null;
		
		// Since the only way to add a TypepParser is adding it with its corresponding Class, nothing can possibly go wrong...
		AbstractTypeParser<T> parser = (AbstractTypeParser<T>)XMLParser.parsers.get(clazz);
		if (parser == null){
			throw new IllegalArgumentException("Type "+clazz.getName()+" is not supported by XMLParser. You can add your own parser for "+clazz.getName()+" using XMLParser.addTypeSupport(Class<T> clazz, TypeParser<T> parser) method."); 
		}
		try{
			result = parser.parse(value);
		}catch (Exception e) {
			result = parser.defaultValueOnException(e);
			LogFactory.getLog(XMLParser.class).error("Parser for type "+clazz.getName()+" failed to parse value '"+value+"' return default value : "+result,e);
		}
		return result;
	}
	
	/**
	 * Add a TypeParser to support or overwrite a Type a Object.
	 * @param clazz the type the parser know how to parse from String
	 * @param parser the parser that transform a String into an Object instance of class clazz.
	 * @throws IllegalClassFormatException if the given Class is of primitive type.
	 */
	public static <T> void  addTypeSupport(Class<T> clazz, AbstractTypeParser<T> parser) throws IllegalClassFormatException{
		if (clazz.isPrimitive()){
			throw new IllegalClassFormatException("Primitive type are not supported by the Annotator parser, please use the equivalent Object.");
		}
		XMLParser.parsers.put(clazz, parser);
	}
	
	/**
	 * By default the following types are supported:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Integer</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>Long</li>
	 * <li>Character</li>
	 * <li>Date</li>
	 * <li>String</li>
	 * </ul>
	 * @return the map of default parsers.
	 */
	public static Map<Class<?>, AbstractTypeParser<?>> resetParsers(){
		XMLParser.parsers = new HashMap<Class<?>, AbstractTypeParser<?>>();
		XMLParser.parsers.put(Double.class, new AbstractTypeParser<Double>() {
			@Override
			public Double parse(String value) {
				return Double.valueOf(value);
			}
		});
		XMLParser.parsers.put(Integer.class, new AbstractTypeParser<Integer>() {
			@Override
			public Integer parse(String value) {
				return Integer.valueOf(value);
			}
		});
		XMLParser.parsers.put(Float.class, new AbstractTypeParser<Float>() {
			@Override
			public Float parse(String value) {
				return Float.valueOf(value);
			}
		});
		XMLParser.parsers.put(Long.class, new AbstractTypeParser<Long>() {
			@Override
			public Long parse(String value) {
				return Long.valueOf(value);
			}
		});
		XMLParser.parsers.put(Character.class, new AbstractTypeParser<Character>() {
			@Override
			public Character parse(String value) {
				return Character.valueOf(value.charAt(0));
			}
		});
		XMLParser.parsers.put(Boolean.class, new AbstractTypeParser<Boolean>() {
			@Override
			public Boolean parse(String value) {
				return Boolean.valueOf(value);
			}
		});
		XMLParser.parsers.put(Byte.class, new AbstractTypeParser<Byte>() {
			@Override
			public Byte parse(String value) {
				return Byte.valueOf(value);
			}
		});
		XMLParser.parsers.put(URI.class, new AbstractTypeParser<URI>() {
			@Override
			public URI parse(String value) {
				return URI.create(value);
			}
		});
		XMLParser.parsers.put(Date.class, new AbstractTypeParser<Date>() {
			@Override
			public Date parse(String value) {
				final DateTimeFormatter parser = ISODateTimeFormat.dateTimeParser();
				return parser.parseDateTime(value).toDate();
			}
		});
		XMLParser.parsers.put(String.class, new AbstractTypeParser<String>() {
			@Override
			public String parse(String value) {
				return value;
			}
		});
		
		return XMLParser.parsers; 
	}

	/**
	 * Transform an object into a Literal.
	 * 
	 * Method is synchronized to prevent from a problem in case of date format.
	 * 
	 * @param value
	 */
	public static synchronized Object xmlTransform(final Object value) {
		if (value instanceof Date) {
			return XMLParser.sdf.format(value);
		}
		return value;
	}

}
