/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 *  
 * Copyright (C) 2004 - 2012 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core.helper.impl;

import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.helper.impl.Rule.Domain;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/**
 * ComplexTripleSelector extends TripleSelectors to provide new way to search in Statements 
 * 
 * @author asaval
 *
 */
public class ComplexTripleSelector extends TripleSelectors implements RDFRulesSelector, AdvancedSelector, org.ow2.weblab.core.helper.impl.SimpleSelector {


	protected transient List<Rule> rules = new LinkedList<Rule>();


	private static final Log LOG = LogFactory.getLog(ComplexTripleSelector.class);


	/**
	 * Creates a ComplexTripleSelector
	 */
	public ComplexTripleSelector() {
	}

	/**
	 *
	 * Creates a ComplexTripleSelector supporting null data and namespaces
	 *
	 * @param supportNullData true to support null data
	 * @param uris list of uris to interpret as resources
	 */
	public ComplexTripleSelector(final boolean supportNullData, final String... namespaces) {
		super(supportNullData, namespaces);
	}


	/**
	 * Add a filter on an association subject/predicat
	 * 
	 * @param subject
	 * @param predicat
	 */
	@Override
	public void addSPRule(final String subject, final String predicat) {
		// filter.addSPFilter(subject, predicat);
		this.rules.add(new Rule(Domain.SUBJECT, subject, Domain.PREDICATE, predicat));
	}


	/**
	 * Add a filter on an association subject/object
	 * 
	 * @param subject
	 * @param object
	 */
	@Override
	public void addSORule(final String subject, final String object) {
		// filter.addSPFilter(subject, object);
		this.rules.add(new Rule(Domain.SUBJECT, subject, Domain.OBJECT, object));
	}


	/**
	 * Add a filter on an association predicat/object
	 * 
	 * @param predicat
	 * @param object
	 */
	@Override
	public void addPORule(final String predicat, final String object) {
		// filter.addSPFilter(predicat, object);
		this.rules.add(new Rule(Domain.PREDICATE, predicat, Domain.OBJECT, object));
	}


	/**
	 * Find all WTriple in a Resource
	 * 
	 * @param resource the Resource
	 * @return a list of WTriple describing RDF annotations in the resource
	 */
	@Override
	public List<WTriple> findIn(final Resource resource) {
		final List<WTriple> list = super.findInResource(resource, null, null, null);
		final Set<WTriple> triples = new LinkedHashSet<WTriple>();
		for (final WTriple wtriple : list) {
			triples.add(wtriple);
		}
		list.clear();
		list.addAll(triples);
		return list;
	}


	/**
	 * Faster
	 */
	@Override
	protected List<WTriple> applySelection(final org.w3c.dom.Node data, final Node subject, final Node predicate, final Node object, final String text, final Resource annotated,
			final List<Resource> resources, final PieceOfKnowledge annotation) {
		if (this.rules.isEmpty()) {
			return super.applySelection(data, subject, predicate, object, text, annotated, resources, annotation);
		}
		final List<WTriple> list = new LinkedList<WTriple>();


		final Model model = TripleSelectors.loadModel(data);
		final Graph graph = model.getGraph();

		final StmtIterator iter = model.listStatements(new SimpleSelector(null, null, (RDFNode) null) {


			@Override
			public boolean selects(final Statement stat) {
				final Triple triple = stat.asTriple();
				for (final Rule rule : ComplexTripleSelector.this.rules) {
					if (ComplexTripleSelector.this.matches(rule, triple)) {
						return true;
					}
				}
				return false;
			}
		});

		while (iter.hasNext()) {
			final Object obj = iter.next();
			if (obj instanceof Statement) {
				final Triple triple = ((Statement) obj).asTriple();
				final Node node = triple.getSubject();
				final WTriple wtriple = new WTriple(triple, text, annotated, this.copyAndAdd(resources, annotation));
				wtriple.setAnnotation(annotation);
				if (node.isBlank()) {
					final ExtendedIterator<?> innerIter = graph.find(null, null, node);
					while (innerIter.hasNext()) {
						final Object innerObject = innerIter.next();
						if (innerObject instanceof Triple) {
							wtriple.setReifiedWTriple(new WTriple((Triple) innerObject, text, annotated, resources));
						}
					}
				}
				list.add(wtriple);
			}
		}
		model.close();
		return list;
	}


	/**
	 * Uses rules
	 */
	@Override
	public Statements searchFor(final Resource resource, final String... predicates) {
		this.rules.clear();
		final Statements res = new Statements(this.uris);
		ComplexTripleSelector.LOG.debug("Resource : search on " + resource);
		if (predicates == null) {
			final List<WTriple> list = this.findInResource(resource, null, null, null, true);
			res.addAll(list);
			return res;
		}

		for (final String pred : predicates) {
			this.addPORule(pred, "(.|\\s)*");
		}
		res.addAll(this.findIn(resource));
		this.rules.clear();
		return res;
	}


	@Override
	public List<WTriple> findInResource(final Resource resource, final Filter filter) {
		this.rules.clear();
		return super.findInResource(resource, filter);
	}


	@Override
	public List<WTriple> findInResource(final Resource resource, final String subject, final String predicate, final String object) {
		this.rules.clear();
		return super.findInResource(resource, subject, predicate, object);
	}


	@Override
	public List<WTriple> findInResource(final Resource resource, final String subject, final String predicate, final String object, final boolean reif) {
		this.rules.clear();
		return super.findInResource(resource, subject, predicate, object, reif);
	}

	/**
	 * Add a rule to the filter. 
	 * @param rule a rule
	 */
	public void addRule(final Rule rule) {
		this.rules.add(rule);
	}


	protected boolean matches(final Rule rule, final Triple triple) {
		final String subject = triple.getSubject().toString();
		final String predicate = triple.getPredicate().toString();
		final String object = triple.getObject().toString();

		String first = null;
		String second = null;
		
		switch (rule.first) {
			case SUBJECT:
				first = subject;
				break;
			case PREDICATE:
				first = predicate;
				break;
			case OBJECT:
			default:
				first = object;
				break;
		}
		if (rule.second != null) {
			switch (rule.second) {
				case SUBJECT:
					second = subject;
					break;
				case PREDICATE:
					second = predicate;
					break;
				case OBJECT:
				default:
					second = object;
					break;
			}
		}
		if (((first != null) && 
				("".equals(rule.firstS) || first.matches("(?i)" + rule.firstS)))) {
			if (second == null) {
				return true;
			}
			if ("".equals(rule.secondS) || second.matches("(?i)" + rule.secondS)) {
				return true;
			}
		}

		return false;
	}


	@Override
	public Results select(final Resource resource, final String... predicates) {
		this.rules.clear();
		final Statements statements = this.searchFor(resource, predicates);
		return new Results(statements);
	}
}
