/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 *  
 * Copyright (C) 2004 - 2012 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.core.helper.impl;

import java.util.LinkedList;
import java.util.Locale;

import junit.framework.Assert;

import org.junit.BeforeClass;
import org.junit.Test;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.factory.WebLabResourceFactory;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.extended.ontologies.RDF;
import org.ow2.weblab.core.extended.ontologies.RDFS;
import org.ow2.weblab.core.helper.impl.Rule.Domain;
import org.ow2.weblab.core.helper.test.ResourcesCreator;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.ComposedResource;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.Text;

import com.hp.hpl.jena.graph.DNode;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;

public class TripleSelectorTest {


	@BeforeClass
	public static void buildResources() throws WebLabCheckedException {
		if (!ResourcesCreator.ANNOTATION.exists()) {
			ResourcesCreator.buildTestResources();
		}
	}

	@Test
	public void testComplexSelector() throws WebLabCheckedException {
		buildResources();
		final ComplexTripleSelector cts = new ComplexTripleSelector();
		final String sujet = "itm:n#_100282";
		final String predicat = RDF.TYPE;
		final String objet = "http://www.weblab-project.org/ontologies/stanag2433.owl#Event";
		cts.addPORule(predicat, objet);
		cts.addSORule(sujet, objet);
		cts.addSPRule(sujet, predicat);

		final Annotation annotation = new WebLabMarshaller().unmarshal(ResourcesCreator.ANNOTATION, Annotation.class);
		Assert.assertEquals(3, cts.findInResource(annotation, null, null, null).size());
		Assert.assertEquals(1, cts.searchFor(annotation).size());
		final Document doc = new WebLabMarshaller().unmarshal(ResourcesCreator.DOCUMENT, Document.class);

		final Rule rule1 = new Rule(Domain.OBJECT, "empty", Domain.SUBJECT, "empty");
		final Rule rule2 = new Rule(Domain.OBJECT, "empty", null, null);
		final ComplexTripleSelector filter = new ComplexTripleSelector();
		filter.addRule(rule1);
		Assert.assertEquals(3, filter.findInResource(annotation, null, null, null).size());
		final ComplexTripleSelector filter2 = new ComplexTripleSelector();
		filter2.addRule(rule2);
		Assert.assertEquals(47, filter2.findInResource(doc, null, null, null).size());
		filter2.limitToFirstLevelAnnotation(true);
		Assert.assertEquals(47, filter2.findInResource(doc, null, null, null).size());
		final TripleSelectors ts = new TripleSelectors();
		Assert.assertEquals(3, ts.findInResource(annotation, null).size());
		Assert.assertEquals(0, ts.findInResource(annotation, "test", "test", "test", true).size());
		
		final ComplexTripleSelector cts2 = new ComplexTripleSelector();
		Assert.assertNotNull(cts2.searchFor(annotation, (String[])null));
		
	}
	
	@Test
	public void testComplexFactory() throws WebLabCheckedException {
		buildResources();
		final RDFRulesSelector rrs1 = RDFSelectorFactory.getSelectorByRules(true, "http://org.weblab-projects/test");
		final Annotation annotation = new WebLabMarshaller().unmarshal(ResourcesCreator.ANNOTATION, Annotation.class);

		rrs1.addSPRule(".*", ".*");
		rrs1.addPORule(".*", ".*");
		rrs1.addSORule(".*", ".*");
		Assert.assertEquals(3, rrs1.findIn(annotation).size());

		final Rule rule1 = new Rule(Domain.OBJECT, "empty", Domain.SUBJECT, "empty");
		final Rule rule2 = new Rule(Domain.OBJECT, "empty", null, null);
		((ComplexTripleSelector) rrs1).rules.add(rule2);
		Assert.assertTrue(((ComplexTripleSelector) rrs1).matches(rule2, new Triple(Node.createURI("empty"), Node.createURI("test"), Node.createURI("empty"))));

		Assert.assertEquals("{OBJECT:empty, SUBJECT:empty}", rule1.toString());

		final AdvancedSelector rs = RDFSelectorFactory.getSelector(true);
		Statements wm = rs.searchFor(annotation);
		Assert.assertEquals(270, wm.toString().length());
		wm = rs.searchFor(annotation, "weblab://property/test");
		Assert.assertEquals(0, wm.size());
		final ComposedResource rc = new ComposedResource();
		rc.setUri("weblab://test/test");
		rc.getResource().add(annotation);

		Assert.assertNull(wm.getResource("weblab://test/test"));
		Assert.assertNull(wm.getAnnotatedResourceWith(annotation.getUri()));

	}


	@Test
	public void testReifComplexSelector() throws WebLabCheckedException {
		buildResources();
		final AdvancedSelector rdfs = RDFSelectorFactory.getSelector(true, "http://marshmalow.gum.net/ontology#");
		final Document doc = new WebLabMarshaller().unmarshal(ResourcesCreator.DOCUMENT, Document.class);
		final Statements wm = rdfs.searchFor(doc, "http://weblab-project.org/core/model/property/processing/temp/luxid/type",
				"http://weblab-project.org/core/model/property/processing/temp/luxid/meta", RDF.PREDICATE, RDF.OBJECT, RDF.SUBJECT, RDF.TYPE, RDFS.LABEL);
		Assert.assertEquals(2, wm.size());
	}

	@Test
	public void testMatches()  {
		ComplexTripleSelector cts = new ComplexTripleSelector(true);
		
		Assert.assertFalse(cts.matches(new Rule(Domain.SUBJECT, null, Domain.OBJECT, null), new Triple(Node.ANY,Node.ANY,Node.ANY)));
		Assert.assertFalse(cts.matches(new Rule(Domain.PREDICATE, "", Domain.SUBJECT, null), new Triple(Node.ANY,Node.ANY,Node.ANY)));
		
		Assert.assertFalse(cts.matches(new Rule(Domain.PREDICATE, null, Domain.SUBJECT, null), new Triple(Node.ANY,new DNode(){
			public String toString(){
				return null;
			}
		},Node.ANY)));
		
		Assert.assertFalse(cts.matches(new Rule(Domain.PREDICATE, "", Domain.SUBJECT, "test"), new Triple(Node.createLiteral("test"),Node.ANY,Node.ANY)));
		
	}
	
	@Test
	public void testTripleSelector() throws WebLabCheckedException  {
		buildResources();
		TripleSelectors.multi = true;
		final TripleSelectors ts = new TripleSelectors(true,"uri://uri");
		Assert.assertTrue(ts.findMT(new PieceOfKnowledge(), null, null, null, null,null, null).isEmpty());
		
		final Annotation annotation = new WebLabMarshaller().unmarshal(ResourcesCreator.ANNOTATION, Annotation.class);
		WebLabResourceFactory.createAndLinkAnnotation(annotation);
		Assert.assertTrue(ts.findMT(annotation, null, null, null, null,annotation, new LinkedList<Resource>()).isEmpty());
		
		TripleSelectors ts2 = new TripleSelectors();
		Assert.assertTrue(ts2.findMT(new PieceOfKnowledge(), null, null, null, null, null, new LinkedList<Resource>()).isEmpty());
		
		TripleSelectors.multi = false;
		
		ts.limit = true;
		Assert.assertTrue(ts.findInResource(annotation,"test",null,"uri://uri#test").isEmpty());
		ts.limit = false;
		
		ComposedResource cr = new ComposedResource();
		cr.getAnnotation().add(annotation);
		cr.setUri("test://test");
		Assert.assertTrue(ts.findInResource(cr,"test",null,"uri://uri#test").isEmpty());
		
		Document doc1 = new Document();
		doc1.setUri("doc://1");
		Text text = new Text();
		text.setUri("text://1");
		Document doc2 = new Document();
		doc2.setUri("doc://2");
		doc2.getAnnotation().add(annotation);
		doc1.getMediaUnit().add(doc2);
		doc1.getMediaUnit().add(text);
		
		
		Assert.assertTrue(ts.findInResource(doc1,"test",null,"uri://uri#test").isEmpty());
	
		Assert.assertTrue(ts.findInResource(doc1,"test",null,"uri://uri#test",false).isEmpty());
		Assert.assertTrue(ts.findInResource(doc1,"test",null,null,false).isEmpty());
		Assert.assertTrue(ts.findInResource(doc1,"test",null,"dummy",false).isEmpty());
		
	}
	
	@Test
	public void testStatments() throws WebLabCheckedException{
		buildResources();
		final AdvancedSelector rdfs = RDFSelectorFactory.getSelector(true);
		final Document doc = new WebLabMarshaller().unmarshal(ResourcesCreator.DOCUMENT, Document.class);
		Statements sts = rdfs.searchFor(doc);
		
		Assert.assertNull(sts.getFirstValue(null, "mypred", null));
		
		Assert.assertEquals("empty29", sts.getFirstValue("http://www.w3.org/1999/02/empty", RDFS.LABEL, null));
		Assert.assertNull(sts.getFirstValue("http://www.w3.org/1999/02/empty", RDFS.LABEL, Locale.FRENCH));
		
		final Annotation annotation = new WebLabMarshaller().unmarshal(ResourcesCreator.ANNOTATION, Annotation.class);
		sts = rdfs.searchFor(annotation);
		AdvancedLinkedHashMap a = (AdvancedLinkedHashMap)sts.get("http://www.w3.org/1999/02/empty");
		a.writeStatement(RDFS.LABEL, "\"leer aber in Deutsch\"@DE");
		Assert.assertEquals("empty", sts.getFirstValue("http://www.w3.org/1999/02/empty", RDFS.LABEL, Locale.FRENCH));
		Assert.assertEquals("leer aber in Deutsch", sts.getFirstValue("http://www.w3.org/1999/02/empty", RDFS.LABEL, Locale.GERMAN));
		
		
	}
	
	
}
