/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core.helper.impl;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.exception.WebLabUncheckedException;
import org.ow2.weblab.core.extended.ontologies.DCTerms;
import org.ow2.weblab.core.extended.ontologies.WebLabProcessing;
import org.ow2.weblab.core.extended.ontologies.WebLabRetrieval;
import org.ow2.weblab.core.extended.util.PoKUtil;
import org.ow2.weblab.core.helper.PoKHelper;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.utils.BackEndJenaHelper;

import com.hp.hpl.jena.rdf.model.Model;

/**
 * This class is the Jena implementation of the {@link PoKHelper} interface.<br />
 * 
 * More explanation are available on the interface description.
 * 
 * @author EADS WebLab Team
 * @date 2008-01-18
 * @see PoKHelper
 */
public class JenaPoKHelper implements PoKHelper {


	/**
	 * The current <code>Annotation</code> object loaded
	 */
	protected PieceOfKnowledge pok;


	/**
	 * The Jena model containing the RDF graph
	 */
	protected BackEndJenaHelper bejh;


	/**
	 * In auto commit mode, remove and create operations are writing into <code>pok</code> each time they are called. When performing a lot's of actions on the
	 * RDF, best practice is to set auto commit mode to <code>false</code> and to commit at the end of the process.
	 */
	protected boolean autoCommit = true;


	/**
	 * Instantiate a new JenaPoKHelper in autoCommit mode. When performing a lot's of operations on the RDF, best practice is to set auto commit mode to
	 * <code>false</code> and to commit at the end of the process. Thus you can use the two parameter constructor.
	 * 
	 * @param pok
	 *            The PieceOfKnwoledge to load inside this helper.
	 * @see #JenaPoKHelper(PieceOfKnowledge, boolean)
	 */
	public JenaPoKHelper(final PieceOfKnowledge pok) {
		this.setPoK(pok);
		this.autoCommit = true;
	}


	/**
	 * @param pok
	 *            The PieceOfKnwoledge to load inside this helper.
	 * @param autoCommit
	 *            In auto commit mode, remove and create operations are writed into <code>pok</code> each time they are called. When performing a lot's of
	 *            operations on the RDF, best practice is to set auto commit mode to <code>false</code> and to commit at the end of the process.
	 */
	public JenaPoKHelper(final PieceOfKnowledge pok, final boolean autoCommit) {
		super();
		this.setPoK(pok);
		this.autoCommit = autoCommit;
	}


	@Override
	public Set<String> getSubjs() {
		return this.bejh.getSubjs();
	}


	@Override
	public Set<String> getPredsOnSubj(final String uriSubj) {
		return this.bejh.getPredsOnSubj(uriSubj);
	}


	@Override
	public List<String> getLitsOnPredSubj(final String uriSubj, final String uriPred) {
		return this.bejh.getLitsOnPredSubj(uriSubj, uriPred);
	}


	@Override
	public List<String> getRessOnPredSubj(final String uriSubj, final String uriPred) {
		return this.bejh.getRessOnPredSubj(uriSubj, uriPred);
	}


	@Override
	public boolean hasLitStat(final String uriSubj, final String uriPred, final String litObj) {
		return this.bejh.hasLitStat(uriSubj, uriPred, litObj);
	}


	@Override
	public boolean hasResStat(final String uriSubj, final String uriPred, final String uriObj) {
		return this.bejh.hasResStat(uriSubj, uriPred, uriObj);
	}


	@Override
	public void createLitStat(final String uriSubj, final String uriPred, final String litObj, final String language) {
		this.bejh.createLitStat(uriSubj, uriPred, litObj, language);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void createLitStat(final String uriSubj, final String uriPred, final String litObj) {
		this.bejh.createLitStat(uriSubj, uriPred, litObj);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void createResStat(final String uriSubj, final String uriPred, final String uriObj) {
		this.bejh.createResStat(uriSubj, uriPred, uriObj);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void removeLitStat(final String uriSubj, final String uriPred, final String litObj) {
		this.bejh.removeLitStat(uriSubj, uriPred, litObj);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void removeResStat(final String uriSubj, final String uriPred, final String uriObj) {
		this.bejh.removeResStat(uriSubj, uriPred, uriObj);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void removeStatsOnSubj(final String uriSubj) {
		this.bejh.removeStatsOnSubj(uriSubj);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void removeStatsWithPred(final String uriPred) {
		this.bejh.removeStatsWithPred(uriPred);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void removeStatsWithLit(final String litObj) {
		this.bejh.removeStatsWithLit(litObj);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public void removeStatsWithRes(final String uriObj) {
		this.bejh.removeStatsWithRes(uriObj);
		if (this.autoCommit) {
			this.commit();
		}
	}


	@Override
	public Set<String> getSubjsOnPred(final String uriPred) {
		return this.bejh.getSubjsOnPred(uriPred);
	}


	@Override
	public Set<String> getSubjsOnPredLit(final String uriPred, final String litObj) {
		return this.bejh.getSubjsOnPredLit(uriPred, litObj);
	}


	@Override
	public Set<String> getSubjsOnPredRes(final String uriPred, final String uriObj) {
		return this.bejh.getSubjsOnPredRes(uriPred, uriObj);
	}


	@Override
	public void setAutoCommitMode(final boolean autoCommitMode) {
		this.autoCommit = autoCommitMode;
	}


	@Override
	public void loadFromRdfXml(final String rdfXml) throws WebLabCheckedException {
		this.bejh = new BackEndJenaHelper(rdfXml);
	}


	@Override
	public String getRdfXml() {
		if ((this.bejh != null) && (this.pok != null)) {
			return this.bejh.getRdfXml();
		}
		return "";
	}


	@Override
	public boolean isDefinedResource(final String uri) {
		return this.bejh.isDefinedResource(uri);
	}


	@Override
	public void commit() {
		if ((this.bejh != null) && (this.pok != null)) {
			final String data = this.getRdfXml();
			try {
				PoKUtil.setPoKData(this.pok, data);
			} catch (final WebLabCheckedException wce) {
				throw new WebLabUncheckedException("Serialisation of Jena " + "model failed. RDF/XML string appears to be " + "invalid: " + data, wce);
			}
		} else {
			throw new WebLabUncheckedException("Annotation was not set.");
		}
	}


	@Override
	public PieceOfKnowledge getPoK() {
		return this.pok;
	}


	@Override
	public void setPoK(final PieceOfKnowledge thePoK) {
		this.pok = thePoK;
		if (this.pok == null) {
			throw new WebLabUncheckedException("No PieceOfKnowledge to load.");
		}
		if (this.bejh != null) {
			this.bejh.close();
		}
		if (this.pok.getData() == null) {
			this.bejh = new BackEndJenaHelper();
		} else {
			final String data = PoKUtil.getPoKData(this.pok);
			try {
				this.loadFromRdfXml(data);
			} catch (final WebLabCheckedException wlce) {
				LogFactory.getLog(this.getClass()).warn("An error occurs loading the pok.");
				LogFactory.getLog(this.getClass()).trace(data);
				this.bejh = new BackEndJenaHelper();
			}
		}
		this.bejh.setNSPrefix(DCTerms.PREFERRED_PREFIX, DCTerms.NAMESPACE);
		this.bejh.setNSPrefix(WebLabProcessing.PREFERRED_PREFIX, WebLabProcessing.NAMESPACE);
		this.bejh.setNSPrefix(WebLabRetrieval.PREFERRED_PREFIX, WebLabRetrieval.NAMESPACE);
	}


	@Override
	public Set<String> getPreds() {
		return this.bejh.getPreds();
	}


	@Override
	public List<String> getLitsOnPredSubj(final String uriSubj, final String uriPred, final String language) {
		return this.bejh.getLitsOnPredSubj(uriSubj, uriPred, language);
	}


	@Override
	public Map<String, String> setNSPrefix(final String prefix, final String uri) {
		return this.bejh.setNSPrefix(prefix, uri);
	}


	@Override
	public Map<String, String> getNSPrefixes() {
		return this.bejh.getNSPrefixes();
	}


	/**
	 * Returns the Jena model.
	 * 
	 * The autoCommit model does not apply to the returned model.
	 * You should consider using it for reading purpose only.
	 * If you write, you should use the commit method on this helper, not on the model.
	 * 
	 * @return The in-memory Jena model that reflects the content of the annotation.
	 */
	public Model getInnerJenaModel() {
		return this.bejh.getModel();
	}

}
